/*
 * sta.h - STA management header file
 *
 * Copyright (C) 2020-2024 IOPSYS Software Solutions AB.
 *
 * See LICENSE file for source code license information.
 *
 */

#ifndef STA_H
#define STA_H

#include <easy/easy.h>

#ifndef MAC_ADDR_HASH
#define MAC_ADDR_HASH(a)    (a[0] ^ a[1] ^ a[2] ^ a[3] ^ a[4] ^ a[5])
#endif

#define STA_HASHTABLE_SIZE	256
#define sta_hash(o)		(MAC_ADDR_HASH(o) & (STA_HASHTABLE_SIZE - 1))


/* STA states */
enum sta_state {
	STA_ASSOCIATED,		/* associated */
	STA_CONNECTED,		/* authorized */
	STA_LOWRCPI,		/* low RCPI */
	STA_STEERING,		/* steer check ongoing */
	STA_STEERED,		/* successfully steered */
	STA_UNKNOWN,		/* no response to steer-request */
	STA_DISCONNECTED,	/* disconnected */
};

/* represents a STA or bSTA */
struct sta {
	uint8_t macaddr[6];
	uint8_t bssid[6];
	bool is_bsta;
	enum sta_state state;

	int associated;
	time_t assoc_time;
	time_t disassoc_time;

	/* est-rate calc timestamp = s->de_sta->tsp - time_delta */
	uint32_t time_delta;

	struct wifi_sta_element *de_sta;

	struct netif_iface *fh; /* the AP interface the sta is connected at */

	struct node *agent; /* valid only when is_bSTA = true */

	bool bcn_response_tmo;
	atimer_t bcn_metrics_timer;
	struct timespec last_bcn_metrics_query;
	int bcn_report_wait_time;	/* in seconds */
	int latest_assoc_cntrl_mid;

#define BTM_RESP_EXP_TIMEOUT 5
	atimer_t btm_req_timer;

	time_t lookup_time;

	struct steer_sta *steer_data;

	struct hlist_node hlist;	/* link in sta_table */
	struct list_head list;		/* link in node */
};

struct sta *cntlr_find_sta(struct hlist_head *table, uint8_t *macaddr);
struct sta *sta_add(struct hlist_head *table, uint8_t *macaddr);
void cntlr_del_sta(struct hlist_head *table, uint8_t *macaddr);

int sta_link_metrics_process(struct sta *s);
void sta_free_bcn_metrics(struct sta *s);
void sta_free_usta_metrics(struct sta *s);
#endif /* STA_H */
